
//			CaseFix.cpp - replaces @CAPS @LOWER and @UPPER to better handle non-ASCII characters.


#define UNICODE					1
#define _UNICODE				1

#include <stdio.h>
#include <tchar.h>

#include <windows.h>

#include "PlugIn.h"
#include "TakeCmd.h"


#define VER_MAJOR				0
#define VER_MINOR				93
#define VER_BUILD				0
#define VER_PATCH				1

#define PLUGIN_NAME				L"CaseFix"

#ifdef _WIN64
#define PLATFORM				L"x64"
#else
#define PLATFORM				L"x86"
#endif

#define MAX_ARG_LEN				8192
#define SHORT_BUF_LEN			1024
#define MAX_FILENAME_LEN		32768

#define QUOTE					0x22

#define ERRHANDLE				GetStdHandle( STD_ERROR_HANDLE )



const wchar_t PluginFeatures[] = L"@caps,@lower,@upper";



#ifdef CASEFIX_HELP

const wchar_t F_CapsHelpText[]		= L"@CAPS[text] \r\n\
@CAPS[\"sep\",text] \r\n\
\n\
Returns text, with the first letter of each word \n\
capitalized.  The list of separator characters is \n\
optional, and must be quoted if present.\n";

const wchar_t F_LowerHelpText[]		= L"@LOWER[text] \r\n\
\n\
Returns text in lowercase.\n";

const wchar_t F_UpperHelpText[]		= L"@UPPER[text] \r\n\
\n\
Returns text in uppercase.\n";

#endif	//	CASEFIX_HELP



int DisplayErrorHeader()
{
	wchar_t shortbuf[MAX_ARG_LEN] = L"%_batch";

	Qprintf( ERRHANDLE, L"%s plugin: ", PLUGIN_NAME );

	ExpandVariables( shortbuf, 1 );
	if ( shortbuf[0] == '0' )
		return 0;

	wcscpy_s( shortbuf, MAX_ARG_LEN, L"%_batchname [%_batchline]  " );
	ExpandVariables( shortbuf, 1 );
	Qprintf( ERRHANDLE, shortbuf );

	return 0;
}



wchar_t GetEscapeChar()
{
	wchar_t temp[SHORT_BUF_LEN]		= L"%@option[EscapeChar]";


	ExpandVariables( temp, 1 );
	
	if ( temp[0] && ( temp[1] == '\0' ) )
		if ( !iswalnum( temp[0] ) )
			return temp[0];

	return '^';
}



DLLExports int WINAPI f_upper( LPTSTR String )
{
	if ( !String )
		return 1;

	CharUpperW( String );
	return 0;
}



DLLExports int WINAPI f_lower( LPTSTR String )
{
	if ( !String )
		return 1;

	CharLowerW( String );
	return 0;
}



DLLExports int WINAPI f_caps( LPTSTR String )
{
	if ( !String )
		return 1;


	wchar_t Separators[SHORT_BUF_LEN]		= L"";
	wchar_t temp[MAX_ARG_LEN]				= L"";

	wchar_t EscapeChar	= GetEscapeChar();
	BOOL UseSeparators	= FALSE;
	BOOL InWord			= FALSE;
	BOOL Escaped		= FALSE;

	wchar_t *Start		= String;
	wchar_t *ch			= String;
	size_t j			= 0;


	while ( iswspace( *Start ) )							//	skip over any leading spaces
		Start++;

	if ( *Start == QUOTE ) {								//	if the first character is a double quote,
		ch = Start + 1;										//	the user has specified a list of separator characters....

		while ( *ch ) {
			if ( !Escaped && ( *ch == EscapeChar ) )		//	escape char:  next character should be escaped
				Escaped = TRUE;
			else if ( !Escaped && ( *ch == QUOTE ) ) {		//	double quotes end the list
				Start = ch + 1;
				UseSeparators = TRUE;
				break;
			}
			else {											//	anything else, add it to the list
				if ( !wcschr( Separators, *ch ) )
					if ( j < ( SHORT_BUF_LEN - 1 ) )
						Separators[j++] = *ch, Separators[j] = '\0';

				Escaped = FALSE;
			}

			ch++;
		}

		if ( UseSeparators ) {								//	if a list of separator characters was specified,
			if ( ( *Start == ',' ) || ( *Start == ';' ) )	//	skip over the following comma
				Start++;

			while ( iswspace( *Start ) )					//	and any following spaces
				Start++;
		}
	}


	j = 0;													//	done parsing any list of separator characters:

	for ( ch = Start; *ch; ch++ ) {							//	loop through characters in text
		if ( !InWord && iswalpha( *ch ) )					//	if it's a letter at the start of the word,
			*ch = (wchar_t) CharUpperW( (LPWSTR) *ch );		//	capitalize it

		if ( UseSeparators )								//	check for end/start of word
			InWord = !wcschr( Separators, *ch );
		else
			InWord = !iswspace( *ch );

		if ( j < ( MAX_ARG_LEN - 1 ) )						//	stash characters in temp buffer
			temp[j++] = *ch, temp[j] = '\0';
	}

	wcscpy_s( String, MAX_ARG_LEN, temp );					//	all done; copy text back to function buffer
	return 0;												//	and exit
}



BOOL APIENTRY DllMain( HANDLE hModule, DWORD  dwReason, LPVOID lpReserved )
{
	switch( dwReason )	{
		case DLL_PROCESS_ATTACH:
		case DLL_THREAD_ATTACH:
		case DLL_THREAD_DETACH:
		case DLL_PROCESS_DETACH:
			break;
	}
	return TRUE;
}



DLLExports LPPLUGININFO WINAPI GetPluginInfo( void )
{
	static PLUGININFO piInfo;

	piInfo.pszDll			= (LPTSTR) PLUGIN_NAME;
	piInfo.pszAuthor		= (LPTSTR) L"Charles Dye";
	piInfo.pszEmail			= (LPTSTR) L"cdye@unm.edu";
	piInfo.pszWWW			= (LPTSTR) L"http://charlesdye.net/plugins/casefix.html";
	piInfo.pszDescription	= (LPTSTR) L"Replaces @CAPS @LOWER and @UPPER to better handle non-ASCII letters.";
	piInfo.pszFunctions		= (LPTSTR) PluginFeatures;
	piInfo.nMajor			= VER_MAJOR;
	piInfo.nMinor			= VER_MINOR;
	piInfo.nBuild			= VER_BUILD;

	return &piInfo;
}



DLLExports BOOL WINAPI InitializePlugin( void )
{
	wchar_t shortbuf[SHORT_BUF_LEN] = L"%[_transient]%[_pipe]%[noloadmsg]";
	ExpandVariables( shortbuf, 1 );

	if ( !wcscmp( shortbuf, L"00" ) )
		Printf( L"%s v%u.%u.%u loaded.\r\n", PLUGIN_NAME, VER_MAJOR, VER_MINOR, VER_BUILD );

	return 0;
}



DLLExports BOOL WINAPI ShutdownPlugin( BOOL bEndProcess )
{
	return 0;
}



#ifdef CASEFIX_HELP

const int NumHelpTexts							= 3;
const wchar_t HelpFeature[NumHelpTexts][11]		= { L"@caps", L"@lower", L"@upper" };
const wchar_t *HelpText[NumHelpTexts]			= { F_CapsHelpText, F_LowerHelpText, F_UpperHelpText };



//		Simpler MessageBox() help, useful for smaller plugins which don't merit a full HTML help system:

DLLExports int WINAPI Help( LPTSTR inString )
{
	wchar_t WindowTitle[SHORT_BUF_LEN]		= L"";


	wsprintf( WindowTitle, L"%s Plugin Help", PLUGIN_NAME );

	for ( int i = 0; i < NumHelpTexts; i++ ) {
		if ( !_wcsicmp( inString, HelpFeature[i] ) ) {
			MessageBox( GetForegroundWindow(), HelpText[i], WindowTitle, MB_OK );
			return 1;
		}
	}

	return 0;
}

#endif	//	CASEFIX_HELP
